/*   
     
Chess-Ec    Copyright (C)   2001-2008 Marc Dechico


ec_hash.h: Memorize and search movements allready simlated.
   
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

To contact the author:	Marc Dechico
Kerfring
29390 SCAER France
tel: 06 37 98 22 54
Mail: ec@chess-ec.org  
    
target of the table:
memorizing three subsequent movement and the note get by the computer
after evaluation of the situation resulting from these three movements

After three movement( depth of three ) the table will be searched to verify 
if a similar situation has already been memorized:
if not, the exploration will continue for the next level and the note will
be memorized.
if a similar situation is found, exploration is avoided by using the note already
computed. 
   

structure of the table:
the arriving part of the movement is used as a hash-index. 
for exemple:
searching for  E2E4 E7E5 D2D3 (<=> D2d3 E7E5 E2E4)
E4(field number 13)  is used as an index (struct st_arr)
from the adresse found in st_arr I'll search in struct st_dep for E2 
if found, I'll search again in st_dep for E5 and so on.
if complet match is found, I get the note and go to the next move at same depth
instead of diving further in the tree.

thread the memory allocated is roughly 
446 Ko for 40 mvt by level
837 Ko for 55 mvt by level
1   Mo for 60 mvt by level 

*/
#include "ec_global.h"
/*************************************************************/
/* external function */
/*************************************************************/


/* These function to free alocated memmory */ 
extern  int mem_lst_ini (struct st_ind **pst_ind);
extern  int mem_lst_add (struct st_ind **pst_ind, void * p_new_bloc);
extern  int mem_lst_fre (struct st_ind **pst_ind);
extern  void * mem_malloc ( unsigned int size);

/*************************************************************/
/* local function */
/*************************************************************/

// debugging
#ifdef TEST
 struct st_hash *pst_hash;
int ex_app();
void print_st_hash();
void print_arr();
void print_dep();
// debug end
#endif

int ini_mem (struct st_hash ** ppst_hash,
	     int i_mvt_qua_ave,int i_inc_per);
int add_mvt_l1(struct st_hash  *pst_hash ,
	       int i_dep, int i_arr, int i_typ_mvt); 
int add_mvt_l2(struct st_hash  *pst_hash ,
	       int i_dep, int i_arr,int i_typ_mvt); 
int add_mvt_l3(struct st_hash  *pst_hash ,
	       int i_dep, int i_arr, int i_typ_mvt); 
int rec_mvt_l3 (struct st_hash *pst_hash,
	     int i_dep1, int i_arr1, int i_typ_mvt1, 
	     int i_dep2, int i_arr2, int i_typ_mvt2, 
	     int i_dep3, int i_arr3, int i_typ_mvt3, 
		int *pi_not,int *pi_typ_pri_lea);

int fre_mem (struct st_hash ** ppst_hash);

/* */
static int ini_mem_arr(struct st_hash * pst_hash);
static int ini_mem_dep(struct st_hash * pst_hash);
static int inc_mem_arr(struct st_hash * pst_hash);
static int inc_mem_dep(struct st_hash *pst_hash);
static int get_nxt_arr (struct st_hash *pst_hash);
static int get_nxt_dep (struct st_hash *pst_hash); 
static int rec_one_mvt (struct st_arr ** pst_arr,
			int  i_dep, int i_arr, int i_typ_mvt);
static int rec_one_mvt_l3 (struct st_arr ** pst_arr,
			int  i_dep, int i_arr, int i_typ_mvt, 
			   int **pi_not,int **pi_typ_pri_lea);

/*************************************************************/
/* local function */
/*************************************************************/

/* The arriving field  - 11  is used as an index 
   in the engine 100 field are used for the simulation 64 + (4*9)filed 
   used for the border,
   but only 64 are useful in this case and we need to spare memory  */
struct st_arr
{
  struct st_dep * pst_dep [64] ;
};

